@@ -12,21 +12,26 @@
 #' @param scale_sm_xifti,scale_sm_FWHM Only applies if \code{scale=="local"} and
 #'  \code{BOLD} represents CIFTI-format data. To smooth the standard deviation
 #'  estimates used for local scaling, provide a \code{"xifti"} object with data
-#'  locations in alignment with \code{BOLD}, as well as the smoothing FWHM 
+#'  locations in alignment with \code{BOLD}, as well as the smoothing FWHM
 #'  (default: \code{2}). If no \code{"xifti"} object is provided (default), do
-#'  not smooth. 
-#' @inheritParams detrend_DCT_Param
+#'  not smooth.
+#' @param scale_sm_xifti_mask For local scaling with smoothing, the data must
+#'  be unmasked to be mapped back to the surface. So if the data are masked,
+#'  provide the mask here.
+#' @inheritParams TR_param
+#' @inheritParams hpf_param
 #'
 #' @return Normalized BOLD data matrix (\eqn{V \times T})
 #'
 #' @export
-#' 
-#' @importFrom fMRItools nuisance_regression dct_bases
+#'
+#' @importFrom fMRItools nuisance_regression dct_bases dct_convert
 #'
 norm_BOLD <- function(
   BOLD, center_rows=TRUE, center_cols=FALSE,
-  scale=c("global", "local", "none"), scale_sm_xifti=NULL, scale_sm_FWHM=2,
-  detrend_DCT=0){
+  scale=c("local", "global", "none"), scale_sm_xifti=NULL, scale_sm_FWHM=2,
+  scale_sm_xifti_mask=NULL,
+  TR=NULL, hpf=.01){
 
   nT <- ncol(BOLD)
   nV <- nrow(BOLD)
@@ -42,17 +47,30 @@ norm_BOLD <- function(
     )
     scale <- "global"
   }
-  scale <- match.arg(scale, c("global", "local", "none"))
-  if (!is.null(scale_sm_xifti)) { 
+  scale <- match.arg(scale, c("local", "global", "none"))
+  if (!is.null(scale_sm_xifti)) {
     if (!requireNamespace("ciftiTools", quietly = TRUE)) {
       stop("Package \"ciftiTools\" needed to work with CIFTI data. Please install it.", call. = FALSE)
     }
     stopifnot(ciftiTools::is.xifti(scale_sm_xifti))
+    if (!is.null(scale_sm_xifti_mask)) {
+      stopifnot(is.vector(scale_sm_xifti_mask) && is.logical(scale_sm_xifti_mask))
+      stopifnot(sum(scale_sm_xifti_mask) == nV)
+    }
   }
   stopifnot(is.numeric(scale_sm_FWHM) && length(scale_sm_FWHM)==1)
-  if (isFALSE(detrend_DCT)) { detrend_DCT <- 0 }
-  stopifnot(is.numeric(detrend_DCT) && length(detrend_DCT)==1)
-  stopifnot(detrend_DCT >=0 && detrend_DCT==round(detrend_DCT))
+  if (is.null(hpf)) { hpf <- 0 }
+  if (is.null(TR)) {
+    if (hpf==.01) {
+      message("Setting `hpf=0` because `TR` was not provided. Either provide `TR` or set `hpf=0` to disable this message.")
+      hpf <- 0
+    } else if (hpf!=0) {
+      stop("Cannot apply `hpf` because `TR` was not provided. Either provide `TR` or set `hpf=0`.")
+    }
+  } else {
+    stopifnot(fMRItools::is_posNum(TR))
+    stopifnot(fMRItools::is_posNum(hpf, zero_ok=TRUE))
+  }
 
   # Center.
   if (center_rows || center_cols) {
@@ -69,13 +87,21 @@ norm_BOLD <- function(
     } else {
       BOLD <- t(BOLD)
     }
-  } 
+  }
 
   # Detrend.
   # [NOTE]: If `center_cols`, columns won't be centered anymore after detrending.
-  if (detrend_DCT > 0) {
+  if (hpf > 0) {
+    nDCT <- round(fMRItools::dct_convert(T_=nT, TR=TR, f=hpf))
+    if (nDCT == 0) {
+      warning("For the low `hpf` and at the data TR and length, the closest number of DCT bases to use for detrending is zero. Using one instead. See `fMRItools::dct_convert`.")
+      nDCT <- 1
+    }
     if (!center_rows) { voxMeans <- rowMeans(BOLD, na.rm=TRUE) }
-    BOLD <- nuisance_regression(BOLD, cbind(1, dct_bases(nT, detrend_DCT)))
+    BOLD <- fMRItools::nuisance_regression(
+      BOLD,
+      cbind(1, fMRItools::dct_bases(nT, nDCT))
+    )
     if (!center_rows) { BOLD <- BOLD + voxMeans }
   }
 
@@ -96,6 +122,12 @@ norm_BOLD <- function(
     # Smooth estimates, if applicable.
     if (!is.null(scale_sm_xifti) && (scale_sm_FWHM != 0)) {
       # Check `scale_sm_xifti` is valid.
+      is_masked <- !is.null(scale_sm_xifti_mask)
+      # Un-mask, if applicable.
+      if (is_masked) {
+        sig <- c(unmask_mat(as.matrix(sig), scale_sm_xifti_mask))
+        nV <- length(sig)
+      }
       if (nV != nrow(scale_sm_xifti)) {
         stop("`scale_sm_xifti` not compatible with `BOLD`: different spatial dimensions.")
       }
@@ -104,6 +136,8 @@ norm_BOLD <- function(
       }
       # Compute and smooth the SD.
       sig <- ciftiTools::newdata_xifti(ciftiTools::select_xifti(scale_sm_xifti, 1), sig)
+      sig <- ciftiTools::move_to_mwall(sig, NA)
+      sig_mask <- do.call(c, sig$meta$cortex$medial_wall_mask)
       sig <- ciftiTools::smooth_xifti(sig, surf_FWHM=scale_sm_FWHM, vol_FWHM=scale_sm_FWHM)
       sig <- c(as.matrix(sig))
     }
@@ -112,4 +146,4 @@ norm_BOLD <- function(
   }
 
   BOLD
-}
\ No newline at end of file
+}
